//  Programm für eine Wort-Uhr für die BRK-Matrix Hardware
//  Nur mit einem EPS8266 Chip ohne den IoT-Brick
//  
//  Unter Verwendung der IoT Library ab Version 1.03


#define BRKclock_PIN 13                                           // Led Pin
#define DS18B20_PIN  14                                           // Datenleitung des DS18B20-Sensors am GPIO14 des EPS8266-Microcontrollers

#include <all_IoT.h>
#include <all_BRKclock.h>
#include <all_DS18B20.h>
#include <all_MQTT.h>                                             // Wenn man MQTT nicht benutzen möchte, einfach diese Zeile auskommentieren


int    TemperatureVal  = 0;
String Temperature     = "";
String Humidity        = "";
String Lauf_Schrift    = "Brick'R'knowledge";

int    counter = 0;
int    m_alt = 0;
int    s_alt = 0;

void BRKclock_ReadTemperature ()
{
  Serial.println("");
  Serial.print("Temperatur-Sensor lesen... ");
  double t = DS18B20_Temperature();
  if (not(isnan(t)))
  {
    Serial.print("Erfolgreich. ");
    TemperatureVal = int(t + BRKclock_temperatureOffset);
    if (t == -127)
    {
      Temperature = "(Sensor nicht gefunden)";
    }
    else
    {
      Serial.print("Temperatur = " + String(t) + ", Offset = " + String(BRKclock_temperatureOffset));
      Temperature = strrealform(t + BRKclock_temperatureOffset, 32, 1, 1, true, false) + "°C"; // 1 Nachkommastelle, auch Null als Nachkommastelle zulassen
    }
  }
  Serial.println("");
}

void setup()
{ 
  t_TerminalInit();                                               // Terminal initialisieren
  Serial.print(t_TerminalClearScreen());
  IoT_Init(false);                                                // EPS8266 initialisieren (ohne OLED/ADC)
  BRKclock_Init();                                                // BRK-Clock initialisieren
  BRKclock_TestMatrix(10);                                        // Alle LEDs mit 50 ms. Verzögerung einschalten
  IoT_WaitKeypress(2000, false);                                  // 2 s warten, in der Zeit kann mit Taster die Konfig zurückgesetzt werden
  Serial.println("");
  Serial.println("");
  if (IoT_Keypress())                                             // Wenn Taster gedrückt wird, Konfiguration im EEPROM löschen
  {
    IoT_EEPROMclear();                                            // Alle Parameter aus dem EEPROM löschen
    IoT_EEPROMupdate();                                           // EEPROM aktualisieren
    Serial.print("BRK-Clock Parameter aus EEPROM gelöscht. ");
    Serial.println("BRK-Clock wird neu gestartet...");
    delay(100);                                                   // 100 ms. warten
    BRKclock_Pixel.clear();                                       // Pixel löschen
    BRKclock_Pixel.show();                                        // Anzeige aktualisieren
    IoT_WaitNoKeypress();                                         // Warten bis der Taster losgelassen wurde
    t_Restart();                                                  // Neustart durchführen
  }
  BRKclock_WLANautoConnect(true);
  IoT_NTPinit();                                                  // NTP-Timeserver initialisieren
  if (not(DS18B20_Init(false)))                                   // DS18B20 Temperatursensor initialisieren
  {
    Serial.println("Temperature Sensor DS18B20 not found.");
  }
  if (BRKclock_ParameterLoad())                                   // Parameter-Block aus EEPROM laden
  {
    Serial.println("BRK-Clock Parameter aus EEPROM geladen.");
    BRKclock_ParameterApply();                                    // Parameter-Block in die aktuelle Konfiguration übernehmen
  }
  else
  {
    Serial.println("BRK-Clock Parameter im EEPROM nicht vorhanden.");
  }

  #if defined(MQTTlibrary)
    String DeviceID = "Brick-" + hexlong(rnd(2147483647));
    Serial.println("Device-ID = " + DeviceID);
    MQTT_Init("iot.allnet.de", 1883, DeviceID, "open", "Allnet"); // iot.allnet.de = IPval("212.18.29.166") ("open",  "Allnet")
    MQTT_Connect("esp/#");
    MQTT_ConnectTrace = false;
    MQTT_EventTrace = false;
  #endif

  IoT_WebServer.on("/", handleRoot);                              // Browser-Handler (siehe unten) für das Stammverzeichnis
  IoT_WebServer.on("/form01", handleForm01);                      // Handler für die Eingabe der Namen in den Eingabefeldern
  IoT_WebServer.on("/form02", handleForm02);                      // Handler für "IoT-Brick neu starten"
  IoT_WebServer.on("/form03", handleForm03);                      // Handler für "IoT-Brick ausschalten"
  IoT_WebServer.on("/form04", handleForm04);                      // Handler für "BRK-Clock Selbsttest"
  IoT_WebServer.on("/form05", handleForm05);                      // Handler für "BRK-Clock Rechtecke"
  IoT_WebServer.on("/form06", handleForm06);                      // Handler für "BRK-Clock Kaleidoskop mit 8 Farben"
  IoT_WebServer.on("/form07", handleForm07);                      // Handler für "BRK-Clock Kaleidoskop mit 16 Farben"
  IoT_WebServer.on("/form08", handleForm08);                      // Handler für "BRK-Clock Laufschrift"
  IoT_WebServer.on("/form09", handleForm09);                      // Handler für "Temperatur anzeigen"
  IoT_WebServer.on("/form10", handleForm10);                      // Handler für "Schlangenlinien"
  IoT_WebServer.on("/form11", handleForm11);                      // Handler für "Weiße Welle über Regenbogen"
  IoT_WebServer.on("/form12", handleForm12);                      // Handler für "Weißes Pulsieren"
  IoT_WebServer.on("/form13", handleForm13);                      // Handler für "Regenbogen wird weiß"
  IoT_WebServer.on("/form14", handleForm14);                      // Handler für "Ambientebeleuchtung"
  IoT_WebServer.on("/form15", handleForm15);                      // Handler für "Regenbogenzyklus"
  IoT_WebServer.on("/form16", handleForm16);                      // Handler für "Regenbogen"
  IoT_WebServer.on("/form17", handleForm17);                      // Handler für "Einstellungen"
  IoT_WebServer.on("/form18", handleForm18);                      // Handler für "Startseite anzeigen"
  IoT_WebServer.on("/form19", handleForm19);                      // Handler für "Dynamische IP-Adresse"
  IoT_WebServer.on("/form20", handleForm20);                      // Handler für "Taster-Einstellung ändern"
  IoT_WebServer.on("/form21", handleForm21);                      // Handler für "Einstellungen speichern"
  IoT_WebServer.on("/form22", handleForm22);                      // Handler für "Standard Einstellungen wiederherstellen"
  IoT_WebServer.on("/form23", handleForm23);                      // Handler für "Dynamische IP-Adresse"
  IoT_WebServer.on("/form24", handleForm24);                      // Handler für "Demos"
  IoT_WebServer.on("/form25", handleForm25);                      // Handler für "MQTT-Werte"
  IoT_WebServer.on("/form26", handleForm26);                      // Handler für "Farbeinstellung"
  IoT_WebServer.begin();                                          // ab jetzt "horcht" der Server auf HTTP-Anfragen
  Serial.println("HTTP server started");
  IoT_WebServer.handleClient();                                   // Bediene die http Anfragen
  BRKclock_SetClockRGBWColor(BRKclock_clockColorRed, BRKclock_clockColorGreen, BRKclock_clockColorBlue, BRKclock_clockColorWhite); // Schriftfarbe
  Serial.println("");
}

void loop()
{
  IoT_Idle();
  int h = hour();
  int m = minute();
  int s = second();
  bool NTPvalid = IoT_NTPvalid();
  
  if ((m_alt != m) && (NTPvalid))                                 // Minute hat sich geändert und NTP-Time wurde empfangen
  {
    if (BRKclock_colorMode == 0)                                  // Farbe jede Minute wechseln
    {
      BRKclock_SetClockRandomColor();                             // Zufällige Farbe erzeugen
    }
    m_alt = m;
    s_alt = s;
    if (BRKclock_temperatureMode == 1)
    {
      BRKclock_PrintTemperature(TemperatureVal, 5000);            // Temperatur anzeigen & 5 Sekunden warten
    }
    BRKclock_International(h, m);
  }
  else if ((s_alt != s) && (NTPvalid))                            // Sekunde hat sich geändert und NTP-Time wurde empfangen
  {
    s_alt = s;
  }
  else
  {
    delay(100);
    IoT_Idle();
  }

  if (IoT_Keypress())                                             // Wenn Taster gedrückt wird, entsprechende Aktion ausführen
  {
    if (BRKclock_buttonMode == 0)                                 // IP-Adresse als Laufschrift anzeigen
    {
      String ip = "IP: " + IoT_WLANaddress(true) + "  GW: " + IoT_WLANgateway(true) + "  NT: " + IoT_WLANsubnet(true);
      if (counter % 10 == 0) // Infos seriell nicht zu oft ausgeben (ca. jede Sekunden, nur wenn der Button gedrückt wird)
      {
        Serial.println(IoT_NTPdatetime() + " (" + IoT_NTPdaylightSavingTime(true) + ")");
        Serial.print("WiFi is ");
        Serial.print(IoT_WLANconnected() ? "connected" : "not connected");
        Serial.println(". Uptime: " + IoT_WLANuptime(true) + " seit " + IoT_WLANstartDatetime());
        Serial.println(ip);                                       // IP-Information
      }
      IoT_WaitNoKeypress();                                       // Warten bis der Taster losgelassen wurde
      BRKclock_Print(ip, 250);                                    // Laufschrift-Text mit IP-Information, 250 ms. Pause
    }
    else if (BRKclock_buttonMode == 1)                            // Temperatur anzeigen
    {
      BRKclock_PrintTemperature(TemperatureVal, 0);               // Temperatur anzeigen & 10 Sekunden warten
    }
    else if (BRKclock_buttonMode == 2)                            // Rechtecke starten
    {
      BRKclock_Pixel.clear();                                     // Pixel löschen
      BRKclock_Pixel.show();                                      // Anzeige aktualisieren
      IoT_WaitNoKeypress();                                       // Warten bis der Taster losgelassen wurde
      while (not(IoT_Keypress()))
      {
        BRKclock_Rectangles();                                    // Rechtecke-Demo
      }
    }
    else if (BRKclock_buttonMode == 3)                            // Kaleidoskop mit 16 Farben starten
    {
      BRKclock_Pixel.clear();                                     // Pixel löschen
      BRKclock_Pixel.show();                                      // Anzeige aktualisieren
      IoT_WaitNoKeypress();                                       // Warten bis der Taster losgelassen wurde
      while (not(IoT_Keypress()))
      {
        BRKclock_Kaleidoscope(8, true);                           // Kaleidoskop mit 16 Farben
      }
    }
    else if (BRKclock_buttonMode == 4)                            // Schlangenlinien starten
    {
      BRKclock_Pixel.clear();                                     // Pixel löschen
      BRKclock_Pixel.show();                                      // Anzeige aktualisieren
      IoT_WaitNoKeypress();                                       // Warten bis der Taster losgelassen wurde
      while (not(IoT_Keypress()))
      {
        BRKclock_SnakeLines(50);                                  // Schlangenlinien
      }
    }
    else if (BRKclock_buttonMode == 5)                            // Weiße Welle über Regenbogen starten
    {
      BRKclock_Pixel.clear();                                     // Pixel löschen
      BRKclock_Pixel.show();                                      // Anzeige aktualisieren
      IoT_WaitNoKeypress();                                       // Warten bis der Taster losgelassen wurde
      while (not(IoT_Keypress()))
      {
        BRKclock_WhiteOverRainbow(20, 75, 5);                     // Weiße Welle über Regenbogen
      }
    }
    else if (BRKclock_buttonMode == 6)                            // Ambientebeleuchtung starten
    {
      BRKclock_Pixel.clear();                                     // Pixel löschen
      BRKclock_Pixel.show();                                      // Anzeige aktualisieren
      IoT_WaitNoKeypress();                                       // Warten bis der Taster losgelassen wurde
      while (not(IoT_Keypress()))
      {
        BRKclock_AmbienceColors(15000);                           // Ambientebeleuchtung, 15 Sekunden Pause
      }
    }
    else if (BRKclock_buttonMode == 7)                            // Regenbogen starten
    {
      BRKclock_Pixel.clear();                                     // Pixel löschen
      BRKclock_Pixel.show();                                      // Anzeige aktualisieren
      IoT_WaitNoKeypress();                                       // Warten bis der Taster losgelassen wurde
      while (not(IoT_Keypress()))
      {
        BRKclock_Rainbow(50);                                     // Regenbogen
      }
    }
    else if (BRKclock_buttonMode == 8)                            // Regenbogenzyklus starten
    {
      BRKclock_Pixel.clear();                                     // Pixel löschen
      BRKclock_Pixel.show();                                      // Anzeige aktualisieren
      IoT_WaitNoKeypress();                                       // Warten bis der Taster losgelassen wurde
      while (not(IoT_Keypress()))
      {
        BRKclock_RainbowCycle(50);                                // Regenbogenzyklus
      }
    }
    IoT_WaitNoKeypress();                                         // Warten bis der Taster losgelassen wurde
    BRKclock_International(hour(), minute());                     // Normale Zeitanzeige einschalten
  }
  else
  {
    if ((counter % 25 == 0) && not(NTPvalid))                     // NTP Event alle 2 Sekunden auslesen wenn noch nicht empfangen
    {
      if (IoT_NTPeventAvail)                                      // Zeitevent-Infos im Terminal ausgeben
      {
        IoT_NTPprintEvent(IoT_NTPcurrentEvent);
        IoT_NTPeventAvail = false;
      }
    }
    else
    {
      if (counter % 100 == 1)                                    // Temperatur alle 10 Sekunden auslesen
      {
        BRKclock_ReadTemperature();
      }
      else
      {
        #if defined(MQTTlibrary)
          int maxMQTT = 100;
          bool success = MQTT_HandleClient();                   // Verbindung aufrecht erhalten, ggf. neu aufbauen und den nächsten Event zulassen
          while ((MQTT_EventAvail) && (maxMQTT > 0))            // Maximal 100 MQTT-Events auf einmal abfragen
          {
            if (MQTT_EventTrace)
            {
              Serial.println("Event verarbeiten #" + str(101 - maxMQTT));
            }
            IoT_Idle();
            MQTT_GetNextEvent();
            if (not(MQTT_AllnetParseEventCurrent()))            // Falls der Event unbekannt ist, diesen ausgeben
            {
              Serial.print("Unbekannter Event: " + str(MQTT_EventCurrentTime) + " ms.");
              Serial.println(", Name: " + cleanasc(MQTT_EventCurrentTopic));
              Serial.println("Wert: " + cleanasc(MQTT_EventCurrentMessage));
            }
            bool success = MQTT_HandleClient();                 // Verbindung aufrecht erhalten, ggf. neu aufbauen und den nächsten Event zulassen
            maxMQTT -= 1;
          }
        #endif
      }
      IoT_Idle();
      IoT_WebServer.handleClient();                             // Bediene die http Anfragen
      IoT_Idle();
      Serial.print("*");                                        // loop() wurde einmal durchlaufen
    }
  }

  IoT_Idle();

  if (BRKclock_alarmMode == 1)                                  // Wecker prüfen
  {
    if ((h == BRKclock_alarmHour) && (m == BRKclock_alarmMinute)) // Weckzeit erreicht?
    {
      byte i = 0;
      while ((i < 60) && not(IoT_Keypress()))                   // 60 Sekunden Alarm oder bis zum Drücken des Tasters
      {
        if (not(IoT_Keypress()))
        {
          BRKclock_Pixel.clear();                               // Pixel löschen
          BRKclock_Pixel.show();                                // Anzeige aktualisieren
          IoT_Idle();
          IoT_WaitKeypress(500, false);                         // 500 ms. warten, Wartezeit wird durch den Button unterbrochen
          BRKclock_International(h, m);
          if (not(IoT_Keypress()))
          {
            long t = millis();
            IoT_WebServer.handleClient();                       // Bediene die http Anfragen
            IoT_Idle();
            t = 500 - (millis() - t);
            if (t > 0)
            {
              IoT_WaitKeypress(t, false);                       // ≤500 ms. warten, Wartezeit wird durch den Button unterbrochen
            }
          }
        }
        i++;
      }
      BRKclock_alarmMode = 2;                                   // Alarm beendet, verhindert dass der Alarm sofort wieder beginnt
    }
  }
  else if (BRKclock_alarmMode == 2)                             // Alarm beendet
  {
    if (not((h == BRKclock_alarmHour) && (m == BRKclock_alarmMinute)))  // Weckzeit vorbei?
    {
      BRKclock_alarmMode = 1;                                   // Normalen Alarm-Modus wiederherstellen
    }
  }

  delay(500);                                                   // 500 ms. warten
  IoT_Idle();
  counter++;
}

// Mit der Funktion handleRoot() wird die Website ausgeliefert sobald eine Anfrage von einem Browser eintrifft

void handleRoot ()
{
  IoT_Idle();
  String time_web = IoT_NTPtime();
  String date_web = IoT_NTPdate();
  String alarm_web = "";
  if (BRKclock_alarmMode == 1)                                    // Wecker eingeschaltet
  {
    alarm_web = strform(BRKclock_alarmHour, 48, 2, false) + ":" + strform(BRKclock_alarmMinute, 48, 2, false) + ":00";
  }
  else
  {
    alarm_web = "(ausgeschaltet)";
  }
  String ipaddr_web  = IoT_WLANaddress(false);
  String title_web = "Brick'R'knowledge BRK-Clock Startseite";
  int sec = millis() / 1000;
  int min = sec / 60;
  int hr = min / 60;
  String ColorHexVal = "";
  String staticInfo = " (dynamisch)";
  if (BRKclock_staticIP == 1)
  {
    staticInfo = " (statisch)";
  }
  switch (BRKclock_colorMode) // Farbe für Web-Interface
  {
    case 0: // Random
      ColorHexVal = "#000000";
      break;
    case 1: // Weiß
      ColorHexVal = "#000000";
      break;
    case 2: // Rot
      ColorHexVal = "#FF0000";
      break;
    case 3: // Grün
      ColorHexVal = "#00FF00";
      break;
    case 4: // Blau
      ColorHexVal = "#0000FF";
      break;
    case 5: // Türkis
      ColorHexVal = "#00FFFF";
      break;
    case 6: // Violett
      ColorHexVal = "#FF00FF";
      break;
    case 7: // Gelb
      ColorHexVal = "#FFFF00";
      break;
    case 8: // Orange
      ColorHexVal = "#FF8000";
      break;
    default: // Unbekannte Auswahl
      ColorHexVal = "#000000";
      break;
  }
  String content =                                                  // Die HTML-Seite in lesbarer Formatierung,
                                                                    // darf auch Variablen enthalten
"<html>\
  <head>\
    <title>" + cleanhtml(title_web) + "</title>\
    <style>\
      body { background-color: #FFFFFF; font-family: Menlo, Monaco, Courier, monospace; Color: " + ColorHexVal + "; }\
    </style>\
    <style type='text/css'>\
      h1 { font-family: Arial, 'Helvetica Neue', Helvetica, sans-serif; Color: #000088; }\
    </style>\
  </head>\
  <body>\
    <h1>" + title_web + "</h1>\
    <p>" + cleanhtml(fillcenter(" Informationen ", "—", 64)) + "</p>\
    <p> </p>\
    <p>" + cleanhtml("Datum:          " + date_web)    + "</p>\
    <p>" + cleanhtml("Uhrzeit:        " + time_web)    + "</p>\
    <p>" + cleanhtml("Weckzeit:       " + alarm_web)   + "</p>\
    <p>" + cleanhtml("Temperatur:     " + Temperature) + "</p>\
    <p>" + cleanhtml("IP-Adresse:     " + ipaddr_web)  + staticInfo + "</p>\
    <p>" + cleanhtml("Öffentliche IP: ") + IoT_WebClientIP() + "</p>\
    <p>" + cleanhtml(fillcenter(" BRK-Clock Farbe & Alarm ", "—", 64)) + "</p>\
    <p> </p>";
content +=
        IoT_WebFormActionButton ("form26", "BRK-Clock Farbe & Alarm") + "\
    <p> </p>\
    <p>" + cleanhtml(fillcenter(" Demos ", "—", 64)) + "</p>\
    <p> </p>\
    " + IoT_WebFormActionButton ("form24", "Demos");
#if defined(MQTTlibrary)
content +=
    "<p>" + cleanhtml(fillcenter(" MQTT-Werte ", "—", 64)) + "</p>\
    <p> </p>\
    " + IoT_WebFormActionButton ("form25", "MQTT-Werte anzeigen");
#endif
content +=
    "<p>" + cleanhtml(fillcenter(" System ", "—", 64)) + "</p>\
    <p> </p>\
    " + IoT_WebFormActionButton ("form17", "System-Einstellungen") + "\
  </body>\
</html>";

  IoT_WebUpdate(&content);                                                   // HTTP-Seite aktualisieren
  content = "";
}

void handleForm01 ()
{
  Serial.println("Button wurde betaetigt: Form01");
  // IoT_WebPrintAllFields();                                          // Alle gefundenen Felder auf dem Terminal ausgeben
  IoT_Idle();
  
  BRKclock_temperatureMode = boolval(IoT_WebTestField("display"));  // Testen, ob die Checkbox Display angekreuzt wurde
  BRKclock_alarmMode   = boolval(IoT_WebTestField("alarm"));        // Testen, ob die Checkbox Wecker angekreuzt wurde
  BRKclock_alarmHour       = val(IoT_WebGetField("wh"));
  BRKclock_alarmMinute     = val(IoT_WebGetField("wm"));

  if (IoT_WebTestField("COLORgroup"))                               // Testen, ob Felder mit dem Namen "COLORgroup" existieren
  {
    String selection = IoT_WebGetField("COLORgroup");
    if (selection == "random")                                      // Schwarzer Text im Web, zufällige Farbe auf der Uhr
    {
      BRKclock_colorMode = 0;                                       // Ab der nächsten Minute die Farbe ändern
      BRKclock_SetClockRandomColor();                               // Zufällige Farbe erzeugen
      BRKclock_SetTemperatureRGBWColor(BRKclock_clockColorRed, BRKclock_clockColorGreen, BRKclock_clockColorBlue, BRKclock_clockColorWhite);
    }
    else if (selection == "black")                                  // Schwarzer Text im Web, weiße Schrift auf der Uhr
    {
      BRKclock_colorMode = 1;
      BRKclock_SetClockRGBWColor(0xFF, 0xFF, 0xFF, 0);
      BRKclock_SetTemperatureRGBWColor(0xFF, 0xFF, 0xFF, 0);
    }
    else if (selection == "red")                                    // Rot
    {
      BRKclock_colorMode = 2;
      BRKclock_SetClockRGBWColor(0xFF, 0x00, 0x00, 0);
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x00, 0x00, 0);
    }
    else if (selection == "green")                                  // Grün
    {
      BRKclock_colorMode = 3;
      BRKclock_SetClockRGBWColor(0x00, 0xFF, 0x00, 0);
      BRKclock_SetTemperatureRGBWColor(0x00, 0xFF, 0x00, 0);
    }
    else if (selection == "blue")                                   // Blau
    {
      BRKclock_colorMode = 4;
      BRKclock_SetClockRGBWColor(0x00, 0x00, 0xFF, 0);
      BRKclock_SetTemperatureRGBWColor(0x00, 0x00, 0xFF, 0);
    }
    else if (selection == "cyan")                                   // Türkis
    {
      BRKclock_colorMode = 5;
      BRKclock_SetClockRGBWColor(0x00, 0xFF, 0xFF, 0);
      BRKclock_SetTemperatureRGBWColor(0x00, 0xFF, 0xFF, 0);
    }
    else if (selection == "magenta")                                // Violett
    {
      BRKclock_colorMode = 6;
      BRKclock_SetClockRGBWColor(0xFF, 0x00, 0xFF, 0);
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x00, 0xFF, 0);
    }
    else if (selection == "yellow")                                 // Gelb
    {
      BRKclock_colorMode = 7;
      BRKclock_SetClockRGBWColor(0xFF, 0xFF, 0x00, 0);
      BRKclock_SetTemperatureRGBWColor(0xFF, 0xFF, 0x00, 0);
    }
    else if (selection == "orange")                                 // Orange
    {
      BRKclock_colorMode = 8;
      BRKclock_SetClockRGBWColor(0xFF, 0x80, 0x00, 0);
      BRKclock_SetTemperatureRGBWColor(0xFF, 0x80, 0x00, 0);
    }
    else if (selection == "user")                                   // Benutzerdefinierte Farbe
    {
      BRKclock_colorMode = 9;
      BRKclock_SetClockRGBWColor      (val(IoT_WebGetField("ur")),
                                       val(IoT_WebGetField("ug")),
                                       val(IoT_WebGetField("ub")),
                                       val(IoT_WebGetField("uw")));
      BRKclock_SetTemperatureRGBWColor(val(IoT_WebGetField("tr")),
                                       val(IoT_WebGetField("tg")),
                                       val(IoT_WebGetField("tb")),
                                       val(IoT_WebGetField("tw")));
    }
    BRKclock_International(hour(), minute());
  }
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  handleForm26();                                                   // Farb-Einstellungen anzeigen
}

void handleForm02 ()                                                // Neustart
{
  Serial.println("Button wurde betaetigt: Form02");
  handleForm17();                                                   // Einstellungen anzeigen
  IoT_Idle();
  BRKclock_Clear(true);                                             // Pixel löschen & Anzeige aktualisieren
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  t_Restart();
}

void handleForm03 ()                                                // Ausschalten
{
  Serial.println("Button wurde betaetigt: Form03");
  handleRoot();                                                     // Wieder auf die Hauptseite zurück schalten
  BRKclock_Clear(true);                                             // Pixel löschen & Anzeige aktualisieren
  IoT_ShutDown();
}

void handleForm04 ()                                                // Selbsttest
{
  Serial.println("Button wurde betaetigt: Form04");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  BRKclock_TestMatrix(50);                                          // Alle LEDs mit 50 ms. Verzögerung einschalten
  IoT_WaitKeypress(0, false);                                       // Auf Taster warten
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm05 ()                                                // Rechtecke Demo
{
  Serial.println("Button wurde betaetigt: Form05");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Rectangles();                                            // Rechtecke-Demo
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm06 ()                                                // Kaleidoskop mit 8 Farben
{
  Serial.println("Button wurde betaetigt: Form06");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Kaleidoscope(8, false);
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm07 ()                                                // Kaleidoskop mit 16 Farben
{
  Serial.println("Button wurde betaetigt: Form07");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Kaleidoscope(8, true);
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm08 ()                                                // Laufschrift
{
  Serial.println("Button wurde betaetigt: Form08");
  Lauf_Schrift = replace(IoT_WebGetField("lauf"), chr(160), " ");   // Geschützes Leerzeichen in Leerzeichen ersetzen
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Print(Lauf_Schrift, 100);                                // Laufschrift-Text von der Webseite laden
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm09 ()                                                // Temperatur anzeigen
{
  Serial.println("Button wurde betaetigt: Form09");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_PrintTemperature(TemperatureVal, 10000);                 // Temperatur anzeigen & 10 Sekunden warten
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm10 ()                                                // Schlangenlinien
{
  Serial.println("Button wurde betaetigt: Form10");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  BRKclock_SnakeLines(50);                                          // Schlangenlinien
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_Pixel.setBrightness(255);
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm11 ()                                                // Weiße Welle über Regenbogen
{
  Serial.println("Button wurde betaetigt: Form11");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  BRKclock_WhiteOverRainbow(20, 75, 5);                             // Weiße Welle über Regenbogen
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_Pixel.setBrightness(255);
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm12 ()                                                // Weißes Pulsieren
{
  Serial.println("Button wurde betaetigt: Form12");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  BRKclock_PulseWhite(5, 4); 
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_Pixel.setBrightness(255);
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm13 ()                                                // Regenbogen wird weiß
{
  Serial.println("Button wurde betaetigt: Form13");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  BRKclock_RainbowFade2White(3,3,1);
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_Pixel.setBrightness(255);
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm14 ()                                                // Ambientebeleuchtung
{
  Serial.println("Button wurde betaetigt: Form14");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  BRKclock_AmbienceColors(1000);                                    // Ambientebeleuchtung
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_Pixel.setBrightness(255);
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm15 ()                                                // Regenbogenzyklus
{
  Serial.println("Button wurde betaetigt: Form15");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  BRKclock_RainbowCycle(50);                                        // Regenbogenzyklus
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_Pixel.setBrightness(255);
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm16 ()                                                // Regenbogen
{
  Serial.println("Button wurde betaetigt: Form16");
  handleForm24();                                                   // Demo-Seite anzeigen
  BRKclock_Clear(false);                                            // Pixel löschen & Anzeige aktualisieren
  BRKclock_Rainbow(50);                                             // Regenbogen
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  BRKclock_Pixel.setBrightness(255);
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm17 ()
{
  Serial.println("Button wurde betaetigt: Form17");
  IoT_Idle();
  String ColorHexVal = "#000000";
  String ipaddr_web  = IoT_WLANaddress(false);
  String ipaddr_gateway  = IoT_WLANgateway(false);
  String ipaddr_subnet  = IoT_WLANsubnet(false);
  String title_web = "Brick'R'knowledge BRK-Clock System";
  String staticInfo = " (dynamisch)";
  if (BRKclock_staticIP == 1)
  {
    staticInfo = " (statisch)";
  }
  int sec = millis() / 1000;
  int min = sec / 60;
  int hr = min / 60;
  String content =                                                  // Die HTML-Seite in lesbarer Formatierung,
                                                                    // darf auch Variablen enthalten
"<html>\
  <head>\
    <title>" + title_web + "</title>\
    <style>\
      body { background-color: #FFFFFF; font-family: Menlo, Monaco, Courier, monospace; Color: " + ColorHexVal + "; }\
    </style>\
    <style type='text/css'>\
      h1 { font-family: Arial, 'Helvetica Neue', Helvetica, sans-serif; Color: #000088; }\
    </style>\
  </head>\
  <body>\
    <h1>" + title_web + "</h1>\
    <p>" + cleanhtml(fillcenter(" Aktuelle Netzwerk-Einstellungen ", "—", 64)) + "</p>\
    <p> </p>";
content +=
   "<p>" + cleanhtml("IP-Addresse   : " + ipaddr_web)      + "</p>\
    <p>" + cleanhtml("Gateway       : " + ipaddr_gateway)  + "</p>\
    <p>" + cleanhtml("Subnetzmaske  : " + ipaddr_subnet)   + "</p>\
    <p> </p>\
    <p>" + cleanhtml("Öffentliche IP: ") + IoT_WebClientIP() + "</p>\
    <p>" + cleanhtml(fillcenter(" Netzwerk ", "—", 64)) + "</p>\
    <p> </p>";
content +=
        IoT_WebFormOpen("form19") +
      "<p>" + cleanhtml("         IP-Adresse     Gateway     Subnetzmaske") + "</p>\
      " + IoT_WebFormSubmitButton("Ändern") + "\
      " + IoT_WebInput(" ", "ip01", ipaddr_web, 15) + "\
      " + IoT_WebInput(" ", "ip02", ipaddr_gateway, 15) + "\
      " + IoT_WebInput(" ", "ip03", ipaddr_subnet, 15) + staticInfo + "\
    " + IoT_WebFormClose() + "\
    " + IoT_WebFormActionButton ("form23", "Dynamische IP-Adresse") + "\
    <p> </p>";
content +=
   "<p>" + cleanhtml(fillcenter(" Sprache & Taster ", "—", 64)) + "</p>\
    <p> </p>\
    " + IoT_WebFormOpen("form20") + "\
      " + IoT_WebInput("Temperatur-Offset +/-: ", "BRKclock_temperatureOffset", strrealform(BRKclock_temperatureOffset, 32, 1, 1, true, false), 5) + cleanhtml(" °C") + "<br>\
      <p> </p>\
      " + IoT_WebRadioButton(" Deutsche Wort-Uhr",            0, "LANGgroup",   "DE",  BRKclock_language)   + "<br>\
      " + IoT_WebRadioButton(" English Word-Clock",           1, "LANGgroup",   "EN",  BRKclock_language)   + "<br>\
      <p> </p>";
content +=
          IoT_WebRadioButton(" IP-Adresse anzeigen",          0, "BTgroup", "butnIP",  BRKclock_buttonMode) + "<br>\
      " + IoT_WebRadioButton(" Temperatur anzeigen",          1, "BTgroup", "butnTP",  BRKclock_buttonMode) + "<br>\
      " + IoT_WebRadioButton(" Rechtecke",                    2, "BTgroup", "butn02",  BRKclock_buttonMode) + "<br>\
      " + IoT_WebRadioButton(" Kaleidoskop",                  3, "BTgroup", "butn03",  BRKclock_buttonMode) + "<br>\
      " + IoT_WebRadioButton(" Schlangenlinien",              4, "BTgroup", "butn04",  BRKclock_buttonMode) + "<br>\
      " + IoT_WebRadioButton(" Weiße Welle über Regenbogen",  5, "BTgroup", "butn05",  BRKclock_buttonMode) + "<br>\
      " + IoT_WebRadioButton(" Ambientebeleuchtung",          6, "BTgroup", "butn06",  BRKclock_buttonMode) + "<br>\
      " + IoT_WebRadioButton(" Regenbogen",                   7, "BTgroup", "butn07",  BRKclock_buttonMode) + "<br>\
      " + IoT_WebRadioButton(" Regenbogenzyklus",             8, "BTgroup", "butn08",  BRKclock_buttonMode) + "<br>\
      <p> </p>";
content +=
          IoT_WebFormSubmitButton("Sprache & Taster-Einstellung ändern") + "<br>\
    " + IoT_WebFormClose() + "\
    <p> </p>";
content +=
   "<p>" + cleanhtml(fillcenter(" Einstellungen ", "—", 64)) + "</p>\
    <p> </p>\
    " + IoT_WebFormActionButton ("form22", "Standard Einstellungen wiederherstellen") + "\
    " + IoT_WebFormActionButton ("form21", "Aktuelle Einstellungen dauerhaft speichern") + "\
    <p> </p>\
    <p>" + cleanhtml(fillcenter(" System ", "—", 64)) + "</p>\
    <p> </p>\
    " + IoT_WebFormActionButton ("form18", "Startseite anzeigen")   + "\
    " + IoT_WebFormActionButton ("form02", "BRK-Clock neu starten") + "\
    " + IoT_WebFormActionButton ("form03", "BRK-Clock ausschalten") + "\
  </body>\
</html>";

  IoT_WebUpdate(&content);                                                   // HTTP-Seite aktualisieren
  content = "";
}

void handleForm18 ()                                                // Startseite anzeigen
{
  Serial.println("Button wurde betaetigt: Form18");
  handleRoot();                                                     // Wieder auf die Hauptseite zurück schalten
  BRKclock_Pixel.setBrightness(255);
  BRKclock_International(hour(), minute());                         // Normale Zeitanzeige einschalten
}

void handleForm19 ()                                                // Statische IP-Adresse
{
  Serial.println("Button wurde betaetigt: Form19");
  BRKclock_staticIP = 1;
  IPAddress ip1 = IPval(IoT_WebGetField("ip01"));                   // Statische IP-Adresse
  IPAddress ip2 = IPval(IoT_WebGetField("ip02"));                   // Gateway/Router IP-Adresse
  IPAddress ip3 = IPval(IoT_WebGetField("ip03"));                   // Subnet IP-Adresse
  WiFi.config(ip1, ip2, ip3);                                       // Parameter: IP, Gateway, Subnet, DNS
  handleForm17();                                                   // Einstellungen anzeigen
}

void handleForm20 ()
{
  Serial.println("Button wurde betaetigt: Form20");
  if (IoT_WebTestField("BTgroup"))                                  // Testen, ob Felder mit dem Namen "BUTTONgroup" existieren
  {
    String selection = IoT_WebGetField("BTgroup");
    if (selection == "butnIP")
    {
      BRKclock_buttonMode = 0;                                      // IP-Adresse anzeigen
    }
    else if (selection == "butnTP")
    {
      BRKclock_buttonMode = 1;                                      // Temperatur anzeigen
    }
    else if (selection == "butn02")
    {
      BRKclock_buttonMode = 2;                                      // Rechtecke
    }
    else if (selection == "butn03")
    {
      BRKclock_buttonMode = 3;                                      // Kaleidoskop
    }
    else if (selection == "butn04")
    {
      BRKclock_buttonMode = 4;                                      // Schlangenlinien
    }
    else if (selection == "butn05")
    {
      BRKclock_buttonMode = 5;                                      // Weiße Welle über Regenbogen
    }
    else if (selection == "butn06")
    {
      BRKclock_buttonMode = 6;                                      // Ambientebeleuchtung
    }
    else if (selection == "butn07")
    {
       BRKclock_buttonMode = 7;                                     // Regenbogen
    }
    else if (selection == "butn08")
    {
      BRKclock_buttonMode = 8;                                      // Regenbogenzyklus
    }
  }
  if (IoT_WebTestField("LANGgroup"))                                // Testen, ob Felder mit dem Namen "LANGgroup" existieren
  {
    String selection = IoT_WebGetField("LANGgroup");
    if (selection == "DE")
    {
      BRKclock_language = 0;                                        // Deutsche Wort-Uhr
      BRKclock_International(hour(), minute());                     // Normale Zeitanzeige einschalten
    }
    else if (selection == "EN")
    {
      BRKclock_language = 1;                                        // Englische Wort-Uhr
      BRKclock_International(hour(), minute());                     // Normale Zeitanzeige einschalten
    }
  }
  BRKclock_temperatureOffset = realval(IoT_WebGetField("BRKclock_temperatureOffset"));              // Temperatur-Offset auslesen
  BRKclock_ReadTemperature();
  IoT_WaitNoKeypress();                                             // Warten bis der Taster losgelassen wurde
  handleForm17();                                                   // Einstellungen anzeigen
}

void handleForm21 ()                                                // Einstellungen speichern (in das EEPROM)
{
  Serial.println("Button wurde betaetigt: Form21");
  BRKclock_ParameterCreate();                                       // Aktuelle Konfiguration in den Parameter-Block schreiben
  BRKclock_ParameterSave();                                         // Parameter-Block in das EEPROM sichern
  IoT_EEPROMupdate();                                               // EEPROM aktualisieren
  handleForm17();                                                   // Einstellungen anzeigen
}

void handleForm22 ()                                                // Standard Einstellungen wiederherstellen
{
  Serial.println("Button wurde betaetigt: Form22");
  BRKclock_ParameterNew();                                          // Aktuelle Konfiguration in den Parameter-Block schreiben
  BRKclock_ParameterApply();                                        // Parameter-Block in die aktuelle Konfiguration übernehmen
  handleForm17();                                                   // Einstellungen anzeigen
}

void handleForm23 ()                                                // Dynamische IP-Adresse
{
  Serial.println("Button wurde betaetigt: Form23");
  BRKclock_staticIP = 0;
  IPAddress ip1 = IPAddress(IoT_dynamicIP_address0, IoT_dynamicIP_address1, IoT_dynamicIP_address2, IoT_dynamicIP_address3);
  IPAddress ip2 = IPAddress(IoT_dynamicIP_gateway0, IoT_dynamicIP_gateway1, IoT_dynamicIP_gateway2, IoT_dynamicIP_gateway3);
  IPAddress ip3 = IPAddress(IoT_dynamicIP_subnet0,  IoT_dynamicIP_subnet1,  IoT_dynamicIP_subnet2,  IoT_dynamicIP_subnet3);
  WiFi.config(ip1, ip2, ip3);                                       // Parameter: IP, Gateway, Subnet, DNS
  handleForm17();                                                   // Einstellungen anzeigen
}

void handleForm24 ()
{
  Serial.println("Button wurde betaetigt: Form24");
  IoT_Idle();
  String ColorHexVal = "#000000";
  String ipaddr_web  = IoT_WLANaddress(false);
  String ipaddr_gateway  = IoT_WLANgateway(false);
  String ipaddr_subnet  = IoT_WLANsubnet(false);
  String title_web = "Brick'R'knowledge BRK-Clock Demos";
  String staticInfo = " (dynamisch)";
  if (BRKclock_staticIP == 1)
  {
    staticInfo = " (statisch)";
  }
  int sec = millis() / 1000;
  int min = sec / 60;
  int hr = min / 60;
  String content =                                                  // Die HTML-Seite in lesbarer Formatierung,
                                                                    // darf auch Variablen enthalten
"<html>\
  <head>\
    <title>" + title_web + "</title>\
    <style>\
      body { background-color: #FFFFFF; font-family: Menlo, Monaco, Courier, monospace; Color: " + ColorHexVal + "; }\
    </style>\
    <style type='text/css'>\
      h1 { font-family: Arial, 'Helvetica Neue', Helvetica, sans-serif; Color: #000088; }\
    </style>\
  </head>\
  <body>\
    <h1>" + title_web + "</h1>\
    <p>" + cleanhtml(fillcenter(" Demos ", "—", 64)) + "</p>\
    <p> </p>";
content +=
        IoT_WebFormActionButton ("form04", "BRK-Clock Selbsttest")
      + IoT_WebFormActionButton ("form09", "BRK-Clock Temperatur")
      + IoT_WebFormActionButton ("form05", "BRK-Clock Rechtecke")
      + IoT_WebFormActionButton ("form06", "BRK-Clock Kaleidoskop mit 8 Farben")
      + IoT_WebFormActionButton ("form07", "BRK-Clock Kaleidoskop mit 16 Farben")
      + IoT_WebFormActionButton ("form10", "BRK-Clock Schlangenlinien");
content +=
        IoT_WebFormActionButton ("form11", "BRK-Clock Weiße Welle über Regenbogen")
      + IoT_WebFormActionButton ("form12", "BRK-Clock Weißes Pulsieren")
      + IoT_WebFormActionButton ("form13", "BRK-Clock Regenbogen wird weiß")
      + IoT_WebFormActionButton ("form14", "BRK-Clock Ambientebeleuchtung")
      + IoT_WebFormActionButton ("form16", "BRK-Clock Regenbogen")
      + IoT_WebFormActionButton ("form15", "BRK-Clock Regenbogenzyklus");
content +=
        IoT_WebFormOpen("form08")
        + IoT_WebFormSubmitButton("BRK-Clock Laufschrift")
        + IoT_WebInput(" ", "lauf", cleanhtml(Lauf_Schrift), 40) + "<br>\
    " + IoT_WebFormClose() + "\
    <p> </p>";
content +=
   "<p>" + cleanhtml(fillcenter(" System ", "—", 64)) + "</p>\
    <p> </p>\
    " + IoT_WebFormActionButton ("form18", "Startseite anzeigen")   + "\
  </body>\
</html>";

  IoT_WebUpdate(&content);                                                   // HTTP-Seite aktualisieren
  content = "";
}

String TemperatureStr (int v)
{
  return (strrealform((double)v / 100.0, 32, 2, 1, false, false) + cleanhtml("°C"));
}

String TemperatureCity (String city, int temperature, int distance)
{
  String result = left(city + spc(13), 13);
  if (position(result, "ä", 1) > 0)         // Umlaute belegen 2 Zeichen im String, daher Leerzeichen zufügen
  {
    result += " ";
  }
  else if (position(result, "ö", 1) > 0)
  {
    result += " ";
  }
  else if (position(result, "ü", 1) > 0)
  {
    result += " ";
  }
  else if (position(result, "ß", 1) > 0)
  {
    result += " ";
  }
  return (cleanhtml(result + "= " + TemperatureStr(temperature) + spc(distance)));
}

void handleForm25 ()
{
  Serial.println("Button wurde betaetigt: Form25");
  IoT_Idle();
  String ColorHexVal = "#000000";
  String title_web = "Brick'R'knowledge BRK-Clock MQTT-Werte";
  String stat = "Server nicht aktiviert";
  String SrvrURL = "(kein Server)";
  #if defined(MQTTlibrary)
    SrvrURL = MQTT_ServerURL;
    if (MQTT_Client.connected())
    {
      stat = "Verbindung hergestellt";
    }
    else
    {
      stat = "Server wird gesucht";
    }
  #endif

  String content =                                                  // Die HTML-Seite in lesbarer Formatierung,
                                                                    // darf auch Variablen enthalten
"<html>\
  <head>\
    <title>" + cleanhtml(title_web) + "</title>\
    <style>\
      body { background-color: #FFFFFF; font-family: Menlo, Monaco, Courier, monospace; Color: " + ColorHexVal + "; }\
    </style>\
    <style type='text/css'>\
      h1 { font-family: Arial, 'Helvetica Neue', Helvetica, sans-serif; Color: #000088; }\
    </style>\
  </head>\
  <body>\
    <h1>" + title_web + "</h1>\
    <p> </p>\
    <p>" + cleanhtml("Verbindungsstatus mit Server '" + SrvrURL + "': " + stat)    + "\
    <p> </p>\
    <p>" + cleanhtml(fillcenter(" Temperaturen ", "—", 80)) + "</p>\
    <p> </p>";
#if defined(MQTTlibrary)
content +=
   "<p>" + TemperatureCity("Hamburg",     MQTT_AllnetTemperaturHamburg,      19) +
           TemperatureCity("Berlin",      MQTT_AllnetTemperaturBerlin,        0) + "</p>\
    <p>" + TemperatureCity("Frankfurt",   MQTT_AllnetTemperaturFrankfurt,    19) +
           TemperatureCity("Stuttgart",   MQTT_AllnetTemperaturStuttgart,     0) + "</p>";
content +=
   "<p>" + TemperatureCity("Hannover",    MQTT_AllnetTemperaturHannover,     19) + \
           TemperatureCity("München",     MQTT_AllnetTemperaturMuenchen,      0) + "</p>\
    <p>" + TemperatureCity("Köln",        MQTT_AllnetTemperaturKoeln,        19) + \
           TemperatureCity("Düsseldorf",  MQTT_AllnetTemperaturDuesseldorf,   0) + "</p>";
content +=
   "<p>" + TemperatureCity("Nürnberg",    MQTT_AllnetTemperaturNuernberg,    19) + \
           TemperatureCity("Dresden",     MQTT_AllnetTemperaturDresden,       0) + "</p>\
    <p>" + TemperatureCity("Naumburg",    MQTT_AllnetTemperaturNaumburg,     19) + \
           TemperatureCity("Heidelberg",  MQTT_AllnetTemperaturHeidelberg,    0) + "</p>";
content +=
   "<p>" + TemperatureCity("Bremen",      MQTT_AllnetTemperaturBremen,       19) + \
           TemperatureCity("Dortmund",    MQTT_AllnetTemperaturDortmund,      0) + "</p>\
    <p>" + TemperatureCity("Kiel",        MQTT_AllnetTemperaturKiel,         19) + \
           TemperatureCity("Leipzig",     MQTT_AllnetTemperaturLeipzig,       0) + "</p>";
content +=
   "<p>" + TemperatureCity("Regensburg",  MQTT_AllnetTemperaturRegensburg,   19) + \
           TemperatureCity("Rostock",     MQTT_AllnetTemperaturRostock,       0) + "</p>\
    <p>" + TemperatureCity("Saarbrücken", MQTT_AllnetTemperaturSaarbruecken, 19) + \
           TemperatureCity("Trier",       MQTT_AllnetTemperaturTrier,         0) + "</p>";
content +=
   "<p>" + TemperatureCity("Ulm",         MQTT_AllnetTemperaturUlm,          19) + \
           TemperatureCity("Würzburg",    MQTT_AllnetTemperaturWuerzburg,     0) + "</p>\
    <p>" + TemperatureCity("Oldenburg",   MQTT_AllnetTemperaturOldenburg,    19) + \
           TemperatureCity("Potsdam",     MQTT_AllnetTemperaturPotsdam,       0) + "</p>";
content +=
   "<p>" + TemperatureCity("Cottbus",     MQTT_AllnetTemperaturCottbus,      19) + \
           TemperatureCity("Schwerin",    MQTT_AllnetTemperaturSchwerin,      0) + "</p>\
    <p>" + TemperatureCity("Mainz",       MQTT_AllnetTemperaturMainz,        19) + \
           TemperatureCity("Wiesbaden",   MQTT_AllnetTemperaturWiesbaden,     0) + "</p>\
    <p> </p>";
#endif
content +=
   "<p>" + cleanhtml(fillcenter(" Aktien- und Währungskurse ", "—", 81)) + "</p>\
    <p> </p>";
#if defined(MQTTlibrary)
content +=
   "<p>" + cleanhtml("DAX         = " + strrealform(MQTT_AllnetFinanceDaxVal,    32, 6, 2, true, false) + spc(16))          + \
           cleanhtml("EUR         = " + strrealform(MQTT_AllnetFinanceUSDVal,    32, 6, 2, true, false) + " USD")           + "</p>\
    <p>" + cleanhtml("MDAX        = " + strrealform(MQTT_AllnetFinanceMDaxVal,   32, 6, 2, true, false)  + spc(16))         + \
           cleanhtml("EUR         = " + strrealform(MQTT_AllnetFinanceCHFVal,    32, 6, 2, true, false) + " CHF")           + "</p>\
    <p>" + cleanhtml("ESTX50      = " + strrealform(MQTT_AllnetFinanceEStx50Val, 32, 6, 2, true, false) + spc(16))          + \
           cleanhtml("EUR         = " + strrealform(MQTT_AllnetFinanceGBPVal,    32, 6, 2, true, false) + " GBP")           + "</p>\
    <p>" + cleanhtml("Gold        = " + strrealform(MQTT_AllnetFinanceGoldVal,   32, 6, 2, true, false) + " EUR" + spc(12)) + \
           cleanhtml("EUR         = " + strrealform(MQTT_AllnetFinanceJPYVal,    32, 6, 2, true, false) + " JPY")           + "</p>\
    <p>" + cleanhtml("Öl          = " + strrealform(MQTT_AllnetFinanceOelVal,    32, 6, 2, true, false) + " EUR" + spc(12)) + \
           cleanhtml("EUR         = " + strrealform(MQTT_AllnetFinanceCNYVal,    32, 6, 2, true, false) + " CNY")           + "</p>\
    <p>" + cleanhtml("Bitcoin     = " + strrealform(MQTT_AllnetFinanceBTCVal,    32, 6, 2, true, false) + " EUR" + spc(12)) + \
           cleanhtml("Ethereum    = " + strrealform(MQTT_AllnetFinanceETHVal,    32, 6, 2, true, false) + " EUR")           + "</p>\
    <p> </p>";
#endif
content +=
   "<p>" + cleanhtml(fillcenter(" Aktualisieren ", "—", 80)) + "</p>";
content +=
   "<p> </p>\
        " + IoT_WebFormActionButton ("form25", "Alle MQTT-Werte aktualisieren")   + "\
    <p> </p>";
content +=
   "<p>" + cleanhtml(fillcenter(" System ", "—", 80)) + "</p>";
content +=
   "<p> </p>\
        " + IoT_WebFormActionButton ("form18", "Startseite anzeigen")   + "\
  </body>\
</html>";

  Serial.println("Web Page Size Form25 = " + str(content.length()) + " Bytes, Free memory = " + str(t_MemAvail()) + " Bytes");
  IoT_WebUpdate(&content);                                                   // HTTP-Seite aktualisieren
  content = "";
}

void handleForm26 ()
{
  IoT_Idle();
  String title_web = "Brick'R'knowledge BRK-Clock Farbe";
  String ColorHexVal = "#000000";
  String content =                                                  // Die HTML-Seite in lesbarer Formatierung,
                                                                    // darf auch Variablen enthalten
"<html>\
  <head>\
    <title>" + cleanhtml(title_web) + "</title>\
    <style>\
      body { background-color: #FFFFFF; font-family: Menlo, Monaco, Courier, monospace; Color: " + ColorHexVal + "; }\
    </style>\
    <style type='text/css'>\
      h1 { font-family: Arial, 'Helvetica Neue', Helvetica, sans-serif; Color: #000088; }\
    </style>\
  </head>\
  <body>\
    <h1>" + title_web + "</h1>\
    <p>" + cleanhtml(fillcenter(" BRK-Clock Farbe & Alarm ", "—", 64)) + "</p>\
    <p> </p>\
    " + IoT_WebFormOpen("form01") + "\
      " + IoT_WebCheckBox(" Wecker eingeschaltet  ", "alarm", "chkd", sgn(BRKclock_alarmMode))
        + IoT_WebInput("H:", "wh", str(BRKclock_alarmHour), 5)
        + IoT_WebInput(" M:", "wm", strform(BRKclock_alarmMinute, 48, 2, false), 5) + "<br>\
      <br>\
      " + IoT_WebCheckBox(" Temperaturanzeige jede Minute für 5 Sekunden", "display", "chkd", BRKclock_temperatureMode) + "<br>\
      <br>";
content +=
          IoT_WebRadioButton(" Zufällige Farbe",  0, "COLORgroup", "random",  BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" Weiße Schrift",    1, "COLORgroup", "black",   BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" Rote Schrift",     2, "COLORgroup", "red",     BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" Grüne Schrift",    3, "COLORgroup", "green",   BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" Blaue Schrift",    4, "COLORgroup", "blue",    BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" Türkise Schrift",  5, "COLORgroup", "cyan",    BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" Violette Schrift", 6, "COLORgroup", "magenta", BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" Gelbe Schrift",    7, "COLORgroup", "yellow",  BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" Orange Schrift",   8, "COLORgroup", "orange",  BRKclock_colorMode) + "<br>\
      " + IoT_WebRadioButton(" RGBW Schrift    ", 9, "COLORgroup", "user",    BRKclock_colorMode);
content +=
          IoT_WebInput("R:", "ur", str(BRKclock_clockColorRed), 5) + IoT_WebInput(" G:", "ug", str(BRKclock_clockColorGreen), 5)
        + IoT_WebInput(" B:", "ub", str(BRKclock_clockColorBlue), 5) + IoT_WebInput(" W:", "uw", str(BRKclock_clockColorWhite), 5) + "<br>\
      " + IoT_WebHtab (26) + cleanhtml("RGBW Temperatur ")
        + IoT_WebInput("R:", "tr", str(BRKclock_tempColorRed), 5) + IoT_WebInput(" G:", "tg", str(BRKclock_tempColorGreen), 5)
        + IoT_WebInput(" B:", "tb", str(BRKclock_tempColorBlue), 5) + IoT_WebInput(" W:", "tw", str(BRKclock_tempColorWhite), 5) + "<br>\
      <br>";
content +=
        IoT_WebFormSubmitButton("Einstellungen der BRK-Clock anwenden") + "<br>\
    " + IoT_WebFormClose() + "\
     <p>" + cleanhtml(fillcenter(" System ", "—", 64)) + "</p>\
    <p> </p>\
        " + IoT_WebFormActionButton ("form18", "Startseite anzeigen")   + "\
  </body>\
</html>";

  IoT_WebUpdate(&content);                                                   // HTTP-Seite aktualisieren
  content = "";
}
